unit uFuenteGaussiana;

interface

uses
  uFichasLPD, xMatDefs, uFechas, fddp,
  uCosa, uGlobs,
  uFuentesAleatorias, SysUtils;

resourcestring
  rsFichaDe = 'Ficha de';
  rsFuenteGaussiana = 'Fuente Gaussiana';

type
(*+doc TFichaFuenteGaussiana
Ficha de parámetros dinámicos para las fuentes Gaussianas
Permite especificar valorEsperado y varianza.
-doc*)

  { TFichaFuenteGaussiana }

  TFichaFuenteGaussiana = class(TFichaLPD)
  public
    valorEsperado, varianza: NReal;
    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      valorEsperado, varianza: NReal);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    class function DescClase: string; override;
    function infoAd_: string; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; virtual;

  end;

(*+doc TFuenteGaussiana
Fuente con distribución gaussiana utilizándo parámetros dinámicos.
-doc*)
  TFuenteGaussiana = class(TFuenteAleatoriaConFichas)
  private
    fgn: Tf_ddp_GaussianaNormal;
  public
    pa: TFichaFuenteGaussiana;

    constructor Create(capa: integer; nombre: string;
      xdurPasoDeSorteoEnHoras: integer; resumirPromediando: boolean; lpd: TFichasLPD);
    class function TipoFichaFuente: TClaseDeFichaLPD; override;
    //function InfoAd : String; override;
    class function DescClase: string; override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure RegistrarParametrosDinamicos; override;
    procedure SortearEntradaRB(var aRB: NReal); override;
    procedure ValorEsperadoEntradaRB(var aRB: NReal); override;

    procedure Free; override;
  end;


procedure AlInicio;
procedure AlFinal;


implementation



//---------------------------------
// Métodos de TFichaFuenteGaussiana
//=================================

constructor TFichaFuenteGaussiana.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; valorEsperado, varianza: NReal);
begin
  inherited Create(capa, fecha, periodicidad);
  self.valorEsperado := valorEsperado;
  self.varianza := varianza;
end;

constructor TFichaFuenteGaussiana.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('valorEsperado', valorEsperado);
  f.rd('varianza', varianza);
  f.EjecutarLectura;
end;

procedure TFichaFuenteGaussiana.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('valorEsperado', valorEsperado, 8, 3);
  f.wr('varianza', varianza, 8, 3);
end;

class function TFichaFuenteGaussiana.DescClase: string;
begin
  Result := rsFichaDe + ' ' + rsFuenteGaussiana;
end;

function TFichaFuenteGaussiana.infoAd_: string;
begin
  Result := 'ValorEsperado= ' + FloatToStrF(valorEsperado, ffGeneral, 10, 2) +
    ', ' + 'Varianza= ' + FloatToStrF(varianza, ffGeneral, 10, 2);
end;

class function TFichaFuenteGaussiana.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
   list: TDataColumnListOfCosa;
begin
  //list:= TDataColumnListOfCosa.Create(TFichaFuenteGaussiana, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('valorEsperado', TFichaFuenteGaussiana(nil).valorEsperado, TDataTypeDoubleOfCosa, 113, -1);
  list.AddDataColumn('varianza', TFichaFuenteGaussiana(nil).varianza, TDataTypeDoubleOfCosa, 113, -1);
  Result := list;
end;

//----------------------------
// Métodos de TFuenteGaussiana
//============================

constructor TFuenteGaussiana.Create(capa: integer; nombre: string;
  xdurPasoDeSorteoEnHoras: integer; resumirPromediando: boolean; lpd: TFichasLPD);

begin
  inherited Create(capa, nombre, xdurPasoDeSorteoEnHoras, resumirPromediando, lpd);
  fgn := nil;
end;

class function TFuenteGaussiana.TipoFichaFuente: TClaseDeFichaLPD;
begin
  Result := TFichaFuenteGaussiana;
end;

{function TFuenteGaussiana.infoAd : String;
begin
  result := 'Gaussiana'
end;}

class function TFuenteGaussiana.DescClase: string;
begin
  Result := rsFuenteGaussiana;
end;

procedure TFuenteGaussiana.prepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
  fgn := Tf_ddp_GaussianaNormal.Create(sorteadorUniforme, 0);
end;

procedure TFuenteGaussiana.RegistrarParametrosDinamicos;
begin
  //Se llama solo despues de preparar memoria que ya expande las fichas
  //no hace falta volver a hacerlo y es un proceso caro si hay periodicidad
  //   lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pa,
    nil, cambioFichaPDFuenteAleatoria);
end;

procedure TFuenteGaussiana.SortearEntradaRB(var aRB: NReal);
begin
  aRB := (fgn.rnd * pa.varianza) + pa.valorEsperado;
end;

procedure TFuenteGaussiana.ValorEsperadoEntradaRB(var aRB: NReal);
begin
  aRB := pa.valorEsperado;
end;


procedure TFuenteGaussiana.Free;
begin
  if fgn <> nil then
    fgn.Free;
  inherited Free;
end;


procedure AlInicio;
begin
  registrarClaseDeCosa(TFuenteGaussiana.ClassName, TFuenteGaussiana);
  registrarClaseDeCosa(TFichaFuenteGaussiana.ClassName, TFichaFuenteGaussiana);
end;

procedure AlFinal;
begin
end;

end.
